@@ -87,22 +87,56 @@ sub module_name {
 
 sub no_index {
     return +{
-      directory => [
-                     'inc',
-                     't',
-                     'xt',
-                   ],
+	directory => [ qw{ eg inc t xt } ],
     };
 }
 
 sub provides {
-    -d 'lib'
-	or return;
+    my $provides;
     local $@ = undef;
-    my $provides = eval {
+
+    eval {
+	require CPAN::Meta;
+	require ExtUtils::Manifest;
 	require Module::Metadata;
-	Module::Metadata->provides( version => 2, dir => 'lib' );
+
+	my $manifest;
+	{
+	    local $SIG{__WARN__} = sub {};
+	    $manifest = ExtUtils::Manifest::maniread();
+	}
+	keys %{ $manifest || {} }
+	    or return;
+
+	# Skeleton so we can use should_index_file() and
+	# should_index_package().
+	my $meta = CPAN::Meta->new( {
+		name	=> 'Euler',
+		version	=> 2.71828,
+		no_index	=> no_index(),
+	    },
+	);
+
+	# The Module::Metadata docs say not to use
+	# package_versions_from_directory() directly, but the 'files =>'
+	# version of provides() is broken, and has been known to be so
+	# since 2014, so it's not getting fixed any time soon. So:
+
+	foreach my $fn ( sort keys %{ $manifest } ) {
+	    $fn =~ m/ [.] pm \z /smx
+		or next;
+	    my $pvd = Module::Metadata->package_versions_from_directory(
+		undef, [ $fn ] );
+	    foreach my $pkg ( keys %{ $pvd } ) {
+		$meta->should_index_package( $pkg )
+		    and $meta->should_index_file( $pvd->{$pkg}{file} )
+		    and $provides->{$pkg} = $pvd->{$pkg};
+	    }
+	}
+
+	1;
     } or return;
+
     return ( provides => $provides );
 }
 
@@ -115,11 +149,10 @@ sub requires {
 	'Encode'	=> 0,
 	'Exporter'	=> 0,
 	'List::Util'	=> 0,
-	'PPI::Document'	=> 1.117,
-	'PPI::Dumper'	=> 1.117,
+	'PPI::Document'	=> 1.238,	# for postfix deref
+	'PPI::Dumper'	=> 1.238,
 	'Readonly'	=> 0,
 	'Scalar::Util'	=> 0,
-	'Text::Tabs'	=> 0,
 	base		=> 0,
 	constant	=> 0,
 	re		=> 0,
@@ -313,7 +346,7 @@ Thomas R. Wyant, III F<wyant at cpan dot org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2016-2021 by Thomas R. Wyant, III
+Copyright (C) 2016-2022 by Thomas R. Wyant, III
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl 5.10.0. For more details, see the full text
